'use strict';

var _extends = require('babel-runtime/helpers/extends')['default'];

var _Promise = require('babel-runtime/core-js/promise')['default'];

var _getIterator = require('babel-runtime/core-js/get-iterator')['default'];

Object.defineProperty(exports, '__esModule', {
    value: true
});
exports.loadAudio = loadAudio;
exports.muted = muted;
exports.setMuted = setMuted;

var _path = require('path');

var _howler = require('howler');

var _load = require('./load');

var _event = require('./event');

var _is = require('./is');

function load(state, eventCode, howlArgs) {
    var sound = new _howler.Howl(howlArgs);

    (0, _event.addHandler)(eventCode, function () {
        sound.play();
    });
    (0, _event.addHandler)(eventCode + '_stop', function () {
        sound.stop();
    });

    return new _Promise(function (resolve, reject) {
        sound.once('loaderror', function (_id, error) {
            reject(error);
        });
        sound.once('load', function () {
            (0, _event.handle)(state, eventCode + '_load_done', sound);
            resolve();
        });
    });
}

// Basic usage: loadAudio(state, 'click', { src: 'click.wav' });
//
// Plays 'click.wav' each time the 'click' event is triggered.
//
// To support multiple formats, pass an array of paths:
//
// loadAudio(state, 'click', { src: ['click.wav', 'click.ogg'] });

function loadAudio(state, eventCode, howlArgs) {
    var src = howlArgs.src;
    if (!(0, _is.isArray)(src)) {
        src = [src];
    }

    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        var _loop = function () {
            var path = _step.value;

            if (path instanceof window.URL) {
                path = path.href;
            }

            var beforeQuery = path.split('?')[0];
            var extension = (0, _path.extname)(beforeQuery).substr(1);
            if (_howler.Howler.codecs(extension)) {
                var _loadData = (0, _load.loadData)(state, path, undefined, 'audio');

                var promise = _loadData.promise;
                var loadingID = _loadData.loadingID;

                promise = promise.then(function () {
                    var args = _extends({}, howlArgs, { src: path, format: [extension] });
                    return load(state, eventCode, args);
                });

                (0, _load.finalizeLoadingPromise)(state, loadingID, promise);
                return {
                    v: undefined
                };
            }
        };

        for (var _iterator = _getIterator(src), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var _ret = _loop();

            if (typeof _ret === 'object') return _ret.v;
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator['return']) {
                _iterator['return']();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    (0, _load.errorLoading)(state, new Error('Unsupported audio format: [' + src + ']'));
}

function muted() {
    return _howler.Howler._muted;
}

function setMuted(muted) {
    _howler.Howler.mute(muted);
}
//# sourceMappingURL=audio.js.map
