'use strict';

var _toConsumableArray = require('babel-runtime/helpers/to-consumable-array')['default'];

var _Object$create = require('babel-runtime/core-js/object/create')['default'];

var _getIterator = require('babel-runtime/core-js/get-iterator')['default'];

Object.defineProperty(exports, '__esModule', {
    value: true
});
exports.addHandler = addHandler;
exports.silence = silence;
exports.handle = handle;

var _util = require('util');

var _is = require('./is');

var handlers = _Object$create(null);

function identity(a) {
    return a;
}

var warned = false;

function addHandler(code, handler) {
    if (process.env.NODE_ENV !== 'production') {
        if (!(0, _is.isFunction)(handler)) {
            throw new Error('handler must be a function');
        }

        if (handling > 0 && !warned) {
            warned = true;
            console.warn('addHandler should only be called during program initialization, ' + 'not after events are triggered');
        }
    }

    var keys = undefined;
    if ((0, _is.isString)(code)) {
        keys = code.split(' ').filter(identity);
    } else if ((0, _is.isArray)(code)) {
        keys = code;
    } else {
        keys = [code];
    }

    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = _getIterator(keys), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var key = _step.value;

            if (key === null || key === 'null' || key === undefined || key === 'undefined') {
                throw new Error('code must be defined');
            }
            var prev = handlers[key];
            if (prev) {
                prev.push(handler);
            } else {
                handlers[key] = [handler];
            }
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator['return']) {
                _iterator['return']();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }
}

var silences = [];

function silence(code) {
    if (process.env.NODE_ENV !== 'production') {
        if ((0, _is.isArray)(code)) {
            silences.push.apply(silences, _toConsumableArray(code));
        } else {
            silences.push(code);
        }
    }
}

var handling = 0;

function handle(state, code, arg) {
    if (process.env.NODE_ENV !== 'production') {
        if (!(0, _is.isHash)(state)) {
            throw new Error('state must be an object');
        }
    }
    if (process.env.NODE_ENV === 'development') {
        // No logging in production or test modes
        if (silences.indexOf(code) < 0) {
            console.log('[event]', code, (0, _util.inspect)(arg, { depth: 0 }));
        }
    }

    var list = handlers[code];
    if (!list) {
        return;
    }
    handling += 1;
    try {
        // for-of syntax gives bad tracebacks
        for (var i = 0; i < list.length; i += 1) {
            var func = list[i];
            func(state, arg);
        }
    } finally {
        handling -= 1;
    }
}
//# sourceMappingURL=event.js.map
