'use strict';

var _createClass = require('babel-runtime/helpers/create-class')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

Object.defineProperty(exports, '__esModule', {
    value: true
});
exports.makeInterpoland = makeInterpoland;

var _array = require('./array');

var _event = require('./event');

var Interpoland = (function () {
    function Interpoland() {
        _classCallCheck(this, Interpoland);
    }

    _createClass(Interpoland, [{
        key: 'mod',
        value: function mod(delta, duration, tweenFunc, onDone) {
            if (!delta && !onDone) {
                return;
            }
            this.dest += delta;
            return makeTween(this.state, this, delta, delta, duration, tweenFunc, onDone);
        }
    }, {
        key: 'modTo',
        value: function modTo(dest, duration, tweenFunc, onDone) {
            return this.mod(dest - this.dest, duration, tweenFunc, onDone);
        }
    }, {
        key: 'modNow',
        value: function modNow(delta) {
            this.base += delta;
            this.curr += delta;
            this.dest += delta;
            if (this.tweenCount === 0) {
                (0, _event.handle)(this.state, 'request_draw');
            }
        }
    }, {
        key: 'modToNow',
        value: function modToNow(dest) {
            this.modNow(dest - this.dest);
        }
    }, {
        key: 'setTo',
        value: function setTo(dest) {
            if (arguments.length > 1) {
                throw new Error('setTo() takes only 1 argument. Did you mean modTo()?');
            }
            if (isNaN(dest)) {
                throw new Error();
            }
            if (this.base === dest && this.tweenCount === 0) {
                return;
            }
            this.base = dest;
            this.curr = dest;
            this.dest = dest;
            if (this.tweenCount === 0) {
                (0, _event.handle)(this.state, 'request_draw');
            } else {
                removeTweensOf(this.state, this);
            }
        }
    }, {
        key: 'mod_noDelta',
        value: function mod_noDelta(amplitude, duration, tweenFunc, onDone) {
            return makeTween(this.state, this, 0, amplitude, duration, tweenFunc, onDone);
        }
    }, {
        key: 'remove',
        value: function remove() {
            removeInterpoland(this.state, this);
        }
    }]);

    return Interpoland;
})();

function removeInterpoland(state, interpoland) {
    var interpolands = state.skid.interpolands;
    (0, _array.remove)(interpolands.interpolands, interpoland);
    removeTweensOf(state, interpoland);
    interpolands.interpolandsUnused.push(interpoland);
}

function removeTweensOf(state, interpoland) {
    if (interpoland.tweenCount === 0) {
        return;
    }
    var interpolands = state.skid.interpolands;
    (0, _array.filter)(interpolands.tweens, function (tween) {
        return tween.interpoland !== interpoland;
    });
    interpoland.tweenCount = 0;
    (0, _event.handle)(state, 'request_draw');
}

function makeTween(state, interpoland, magnitude, amplitude, duration, func, onDone) {
    var interpolands = state.skid.interpolands;
    if (!func) {
        throw new Error();
    }
    if (isNaN(magnitude)) {
        throw new Error();
    }
    if (isNaN(amplitude)) {
        throw new Error();
    }
    if (isNaN(duration)) {
        throw new Error();
    }
    if (isNaN(interpolands.remainder)) {
        throw new Error();
    }
    var result = {
        interpoland: interpoland,
        magnitude: magnitude,
        amplitude: amplitude,
        duration: duration,
        func: func,
        onDone: onDone,
        curr: 0,
        elapsed: interpolands.remainder
    };
    interpolands.tweens.push(result);
    interpoland.tweenCount += 1;
    (0, _event.handle)(state, 'request_draw');
    return result;
}

function makeInterpoland(state, value) {
    if (!state.skid.interpolands) {
        state.skid.interpolands = {
            tweens: [],
            tweensEnding: [],
            interpolands: [],
            interpolandsUnused: [],
            // TODO: merge with state.skid.timeRemainder; need to modify animationFrame
            remainder: 0
        };
    }
    var interpolands = state.skid.interpolands;
    var result = undefined;
    if (interpolands.interpolandsUnused.length) {
        result = interpolands.interpolandsUnused.pop();
    } else {
        result = new Interpoland();
    }
    value = value || 0;
    result.curr = value;
    result.base = value;
    result.dest = value;
    result.state = state;
    result.tweenCount = 0;

    interpolands.interpolands.push(result);
    return result;
}

function updateTween(tween, dt) {
    tween.elapsed += dt;

    if (tween.elapsed >= tween.duration) {
        tween.interpoland.curr += tween.magnitude;
        tween.interpoland.base += tween.magnitude;
        tween.interpoland.tweenCount -= 1;
        return false;
    }

    tween.interpoland.curr += tween.amplitude * tween.func.call(undefined, tween.elapsed / tween.duration);
    return true;
}

(0, _event.addHandler)('before_draw', function (state, dt) {
    var interpolands = state.skid.interpolands;
    if (!interpolands) {
        return;
    }

    for (var i = 0; i < interpolands.interpolands.length; i += 1) {
        interpolands.interpolands[i].curr = interpolands.interpolands[i].base;
    }

    (0, _array.filter)(interpolands.tweens, updateTween, interpolands.tweensEnding, dt);

    for (var i = 0; i < interpolands.tweensEnding.length; i += 1) {
        var tween = interpolands.tweensEnding[i];
        if (!tween.onDone) {
            continue;
        }
        interpolands.remainder = tween.elapsed - tween.duration;
        tween.onDone.call();
    }
    interpolands.tweensEnding.length = 0;
    interpolands.remainder = 0;
});

(0, _event.addHandler)('after_draw', function (state) {
    if (state.skid.interpolands && state.skid.interpolands.tweens.length) {
        (0, _event.handle)(state, 'request_draw');
    }
});
//# sourceMappingURL=interpolands.js.map
