'use strict';

var _Object$create = require('babel-runtime/core-js/object/create')['default'];

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var _getIterator = require('babel-runtime/core-js/get-iterator')['default'];

var _Object$values = require('babel-runtime/core-js/object/values')['default'];

var _Promise = require('babel-runtime/core-js/promise')['default'];

Object.defineProperty(exports, '__esModule', {
    value: true
});
exports.start = start;
exports.startLoading = startLoading;
exports.progressLoading = progressLoading;
exports.doneLoading = doneLoading;
exports.errorLoading = errorLoading;
exports.reloadData = reloadData;
exports.loadData = loadData;
exports.contentTypeMatches = contentTypeMatches;
exports.finalizeLoadingPromise = finalizeLoadingPromise;

var _event = require('./event');

var _is = require('./is');

var started = false;

function start() {
    if (process.env.NODE_ENV !== 'production') {
        if (arguments.length > 0) {
            throw new Error('start() takes no arguments');
        }
    }

    if (started) {
        throw new Error('call start() only once');
    }
    started = true;
    var state = _Object$create(null);
    state.skid = _Object$create(null);
    if (typeof window !== 'undefined') {
        window.getState = function () {
            return state;
        };
    }
    state.skid.load = { requests: 0, completions: 0, loaders: {}, error: false };

    if (typeof window !== 'undefined') {
        window.addEventListener('load', function () {
            doLoad(state);
        });
    } else {
        doLoad(state);
    }
}

function doLoad(state) {
    (0, _event.handle)(state, 'load');
    if (state.skid.load.requests === 0) {
        state.skid.load = undefined;
        (0, _event.handle)(state, 'load_done');
    }
}

function startLoading(state, size) {
    if (!state.skid.load) {
        throw new Error("load assets during the 'load' event");
    }
    state.skid.load.requests += 1;
    var id = state.skid.load.requests;
    if (size != null) {
        progressLoading(state, id, 0, size);
    }
    return id;
}

function progressLoading(state, id, loaded, total) {
    if (!state.skid.load) {
        throw new Error("load assets during the 'load' event");
    }
    if (state.skid.load.error) {
        return;
    }
    if (isNaN(loaded) || isNaN(total)) {
        loaded = 0;
        total = 0;
    }
    if (!state.skid.load.loaders[id]) {
        state.skid.load.loaders[id] = {};
    }
    state.skid.load.loaders[id].loaded = loaded;
    state.skid.load.loaders[id].total = total;

    if (_Object$keys(state.skid.load.loaders).length < state.skid.load.requests) {
        return;
    }

    var allLoaded = 0;
    var allTotal = 0;
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = _getIterator(_Object$values(state.skid.load.loaders)), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var loader = _step.value;

            allLoaded += loader.loaded;
            allTotal += loader.total;
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator['return']) {
                _iterator['return']();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    var progress = allLoaded / allTotal;
    if (allTotal === 0) {
        progress = 0;
    }
    (0, _event.handle)(state, 'load_progress', progress);
}

function doneLoading(state, id) {
    if (!state.skid.load) {
        throw new Error("load assets during the 'load' event");
    }
    if (state.skid.load.error) {
        return;
    }
    state.skid.load.completions += 1;
    if (state.skid.load.completions === state.skid.load.requests) {
        state.skid.load = undefined;
        (0, _event.handle)(state, 'load_done');
    }
}

function errorLoading(state, error) {
    if (state.skid.load.error) {
        return;
    }
    state.skid.load.error = true;
    (0, _event.handle)(state, 'load_error', error);
}

function reloadData(state, url, contentType) {
    return doXHR(state, undefined, url, false, contentType);
}

function loadData(state, url, total, contentType) {
    var loadingID = startLoading(state, total);
    return { promise: doXHR(state, loadingID, url, true, contentType), loadingID: loadingID };
}

function contentTypeMatches(expected, actual) {
    if (!expected.includes(';')) {
        actual = actual.split(';')[0];
    }
    if (!expected.includes('/')) {
        actual = actual.split('/')[0];
    }
    return expected === actual;
}

function doXHR(state, id, url, showProgress, contentType) {
    var promise = new _Promise(function (resolve, reject) {
        var xhr = new XMLHttpRequest();
        xhr.open('GET', url, true);
        xhr.responseType = 'arraybuffer';

        if (showProgress) {
            // just need nonzero placeholder until actual byte count is known
            progressLoading(state, id, 0, 1);

            xhr.onprogress = function (event) {
                if (event.lengthComputable) {
                    progressLoading(state, id, event.loaded, event.total);
                }
            };
        }

        xhr.onloadend = function () {
            var blob = xhrToBlob(xhr);
            if (!blob) {
                reject(new Error('Failed to load ' + url));
                return;
            }

            if (contentType && blob.type) {
                if (!contentTypeMatches(contentType, blob.type)) {
                    // Parcel silently redirects to the root HTML page if it can't resolve the URL
                    // instead of generating an error.

                    reject(new Error('Failed to load ' + url + ': expected content type ' + ('\'' + contentType + '\', got \'' + blob.type + '\'')));
                    return;
                }
            }

            if (showProgress) {
                progressLoading(state, id, blob.size, blob.size);
            }

            var data = window.URL.createObjectURL(blob);
            resolve(data);
        };

        // Explicitly moving to next turn of event loop because some errors will happen
        // synchronously and Skid needs all outputs from this operation to be asynchronous.
        setTimeout(function () {
            return xhr.send();
        });
    });
    return promise;
}

function finalizeLoadingPromise(state, loadingID, promise) {
    promise.then(function () {
        if ((0, _is.isDefined)(loadingID)) {
            doneLoading(state, loadingID);
        }
    })['catch'](function (error) {
        errorLoading(state, error);
        throw error; // makes it show up in the console
    });
}

function xhrToBlob(xhr) {
    if (xhr.status.toString().match(/^2/)) {
        var options = {};
        var headers = xhr.getAllResponseHeaders();
        var match = headers.match(/^Content-Type:\s*(.*?)$/mi);

        if (match && match[1]) {
            options.type = match[1];
        }

        return new Blob([xhr.response], options);
    }
    return undefined;
}
//# sourceMappingURL=load.js.map
