'use strict';

var _get = require('babel-runtime/helpers/get')['default'];

var _inherits = require('babel-runtime/helpers/inherits')['default'];

var _createClass = require('babel-runtime/helpers/create-class')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _slicedToArray = require('babel-runtime/helpers/sliced-to-array')['default'];

Object.defineProperty(exports, '__esModule', {
    value: true
});

var _group = require('./group');

var _translation = require('./translation');

var _scalement = require('./scalement');

var Cache = (function (_Group) {
    _inherits(Cache, _Group);

    function Cache(container, tileSize, imageSmoothingEnabled) {
        _classCallCheck(this, Cache);

        _get(Object.getPrototypeOf(Cache.prototype), 'constructor', this).call(this, container);

        // must set to true when altering any field
        this.altered = false;

        this.imageSmoothingEnabled = imageSmoothingEnabled;

        // affects granularity of saved image data but not final size/position
        this.tileSize = tileSize;

        // Don't directly change these fields from external code:

        this._canvas = document.createElement('canvas');
        this._context = this._canvas.getContext('2d');
        this._x = 0;
        this._y = 0;
        this._width = 0;
        this._height = 0;
    }

    _createClass(Cache, [{
        key: 'insert',
        value: function insert(avatar) {
            _get(Object.getPrototypeOf(Cache.prototype), 'insert', this).call(this, avatar);
            this.altered = true;
        }
    }, {
        key: 'removeAvatar',
        value: function removeAvatar(avatar) {
            _get(Object.getPrototypeOf(Cache.prototype), 'removeAvatar', this).call(this, avatar);
            this.altered = true;
        }
    }, {
        key: 'draw',
        value: function draw(context) {
            var _this = this;

            if (this.altered) {
                this.altered = false;

                var bounds = this.bounds();
                if (bounds) {
                    var _bounds = _slicedToArray(bounds, 4);

                    this._x = _bounds[0];
                    this._y = _bounds[1];
                    this._width = _bounds[2];
                    this._height = _bounds[3];

                    this._canvas.width = Math.ceil(this._width * this.tileSize);
                    this._canvas.height = Math.ceil(this._height * this.tileSize);
                } else {
                    this._canvas.width = 0;
                    this._canvas.height = 0;
                }

                if (!this._canvas.width || !this._canvas.height) {
                    return;
                }

                if (this.imageSmoothingEnabled !== undefined) {
                    this._context.imageSmoothingEnabled = this.imageSmoothingEnabled;
                    this._context.mozImageSmoothingEnabled = this.imageSmoothingEnabled;
                }

                // Scale and translate so draw operations fit in the canvas
                _scalement.Scalement.draw(this._context, this.tileSize, this.tileSize, function () {
                    _translation.Translation.draw(_this._context, -_this._x, -_this._y, function () {
                        // Draw all contents to the saved canvas. The scale/translate operations make it
                        // so that no changes to the contained avatars are necessary.
                        _get(Object.getPrototypeOf(Cache.prototype), 'draw', _this).call(_this, _this._context);
                    });
                });
            } else if (!this._canvas.width || !this._canvas.height) {
                return;
            }

            // Draw the entire saved canvas to the rectangle specified by this.bounds() earlier.
            context.drawImage(this._canvas, 0, 0, this._canvas.width, this._canvas.height, this._x, this._y, this._width, this._height);
        }
    }]);

    return Cache;
})(_group.Group);

exports.Cache = Cache;
//# sourceMappingURL=cache.js.map
