'use strict';

var _createClass = require('babel-runtime/helpers/create-class')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _Object$create = require('babel-runtime/core-js/object/create')['default'];

var _getIterator = require('babel-runtime/core-js/get-iterator')['default'];

Object.defineProperty(exports, '__esModule', {
    value: true
});
exports.borderIcons = borderIcons;

var _tilefield = require('./tilefield');

var _iconavatar = require('./iconavatar');

var _is = require('../is');

var ComputedTileField = (function () {
    function ComputedTileField(root, tileSize, imageSmoothingEnabled) {
        _classCallCheck(this, ComputedTileField);

        this.field = new _tilefield.TileField(root, tileSize, imageSmoothingEnabled);
        this.avatars = _Object$create(null); // {position: {[type]: [avatar, ...]}}
    }

    _createClass(ComputedTileField, [{
        key: '_hasTypeAtKey',
        value: function _hasTypeAtKey(key, types) {
            var map = this.avatars[key];
            if (!map) {
                return false;
            }
            if ((0, _is.isIterable)(types)) {
                var _iteratorNormalCompletion = true;
                var _didIteratorError = false;
                var _iteratorError = undefined;

                try {
                    for (var _iterator = _getIterator(types), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                        var type = _step.value;

                        if (map[type] && map[type].length) {
                            return true;
                        }
                    }
                } catch (err) {
                    _didIteratorError = true;
                    _iteratorError = err;
                } finally {
                    try {
                        if (!_iteratorNormalCompletion && _iterator['return']) {
                            _iterator['return']();
                        }
                    } finally {
                        if (_didIteratorError) {
                            throw _iteratorError;
                        }
                    }
                }
            } else if (map[types] && map[types].length) {
                return true;
            }
            return false;
        }
    }, {
        key: '_addAvatar',
        value: function _addAvatar(x, y, type, avatar) {
            var key = keyOf(x, y);
            var map = this.avatars[key];
            if (!map) {
                map = _Object$create(null);
                this.avatars[key] = map;
            }
            var list = map[type];
            if (!list) {
                list = [];
                map[type] = list;
            }
            list.push(avatar);
        }
    }, {
        key: '_changedAt',
        value: function _changedAt(x, y) {
            var key = keyOf(x, y);
            var lists = this.avatars[key];
            for (var type in lists) {
                var list = lists[type];
                var _iteratorNormalCompletion2 = true;
                var _didIteratorError2 = false;
                var _iteratorError2 = undefined;

                try {
                    for (var _iterator2 = _getIterator(list), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                        var avatar = _step2.value;

                        if (avatar.updater) {
                            avatar.icon = avatar.updater(this, x, y);
                        }
                    }
                } catch (err) {
                    _didIteratorError2 = true;
                    _iteratorError2 = err;
                } finally {
                    try {
                        if (!_iteratorNormalCompletion2 && _iterator2['return']) {
                            _iterator2['return']();
                        }
                    } finally {
                        if (_didIteratorError2) {
                            throw _iteratorError2;
                        }
                    }
                }
            }
            var node = this.field._segments[key];
            if (node) {
                node._altered = true;
            }
        }
    }, {
        key: '_changedAround',
        value: function _changedAround(x, y) {
            this._changedAt(x - 1, y - 1);
            this._changedAt(x, y - 1);
            this._changedAt(x + 1, y - 1);
            this._changedAt(x - 1, y);
            this._changedAt(x + 1, y);
            this._changedAt(x - 1, y + 1);
            this._changedAt(x, y + 1);
            this._changedAt(x + 1, y + 1);
        }
    }, {
        key: '_makeTile4Part',
        value: function _makeTile4Part(icon, x, y, layer, type, sx, sy) {
            var segment = this.field.nodeAt(x, y);
            var avatar = new _iconavatar.IconAvatar(segment, undefined, x + 0.25 * sx, y + 0.25 * sy, 0.5, 0.5);
            avatar.layer = layer;
            if ((0, _is.isFunction)(icon)) {
                avatar.updater = icon;
            } else {
                avatar.icon = icon;
            }
            this._addAvatar(x, y, type, avatar);
        }
    }, {
        key: 'makeTile4',
        value: function makeTile4(_ref, x, y, layer, type) {
            var nw = _ref.nw;
            var ne = _ref.ne;
            var sw = _ref.sw;
            var se = _ref.se;

            this._makeTile4Part(sw, x, y, layer, type, -1, 1);
            this._makeTile4Part(se, x, y, layer, type, 1, 1);
            this._makeTile4Part(nw, x, y, layer, type, -1, -1);
            this._makeTile4Part(ne, x, y, layer, type, 1, -1);
            this._changedAt(x, y);
            this._changedAround(x, y);
        }
    }, {
        key: 'makeTile',
        value: function makeTile(icon, x, y, layer, type) {
            var avatar = new _iconavatar.IconAvatar(this.field.nodeAt(x, y), icon, x, y, 1, 1);
            avatar.layer = layer;
            this._addAvatar(x, y, type, avatar);
            this._changedAround(x, y);
        }
    }, {
        key: 'hasTile',
        value: function hasTile(type, x, y) {
            var lists = this.avatars[keyOf(x, y)];
            return lists && lists[type] && lists[type].length > 0;
        }
    }, {
        key: 'insertAvatar',
        value: function insertAvatar(avatar, x, y, type) {
            this.field.nodeAt(x, y).insert(avatar);
            this._addAvatar(x, y, type, avatar);
            this._changedAround(x, y);
        }
    }, {
        key: 'removeTile',
        value: function removeTile(x, y, type) {
            var lists = this.avatars[keyOf(x, y)];
            if (lists && lists[type]) {
                var _iteratorNormalCompletion3 = true;
                var _didIteratorError3 = false;
                var _iteratorError3 = undefined;

                try {
                    for (var _iterator3 = _getIterator(lists[type]), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
                        var avatar = _step3.value;

                        avatar.remove();
                    }
                } catch (err) {
                    _didIteratorError3 = true;
                    _iteratorError3 = err;
                } finally {
                    try {
                        if (!_iteratorNormalCompletion3 && _iterator3['return']) {
                            _iterator3['return']();
                        }
                    } finally {
                        if (_didIteratorError3) {
                            throw _iteratorError3;
                        }
                    }
                }

                delete lists[type];
                this._changedAt(x, y);
                this._changedAround(x, y);
            }
        }
    }, {
        key: 'clear',
        value: function clear() {
            this.field.clear();
            this.avatars = _Object$create(null);
        }
    }]);

    return ComputedTileField;
})();

exports.ComputedTileField = ComputedTileField;

function keyOf(x, y) {
    return x + ',' + y;
}

function borderIcons(observedTypes, inverse, nw, n, ne, w, c, e, sw, s, se, concaveNW, concaveNE, concaveSW, concaveSE) {
    return {
        nw: selected(-1, -1, observedTypes, inverse, nw, concaveSE, n, w, c),
        ne: selected(1, -1, observedTypes, inverse, ne, concaveSW, n, e, c),
        sw: selected(-1, 1, observedTypes, inverse, sw, concaveNE, s, w, c),
        se: selected(1, 1, observedTypes, inverse, se, concaveNW, s, e, c)
    };
}

function selected(dx, dy, observedTypes, inverse, convex, concave, hFace, vFace, surrounded) {
    return function (cfield, x, y) {
        var cornerKey = keyOf(x + dx, y + dy);
        var hKey = keyOf(x, y + dy);
        var vKey = keyOf(x + dx, y);

        var corner = cfield._hasTypeAtKey(cornerKey, observedTypes);
        var horiz = cfield._hasTypeAtKey(hKey, observedTypes);
        var vert = cfield._hasTypeAtKey(vKey, observedTypes);

        if (inverse) {
            corner = !corner;
            horiz = !horiz;
            vert = !vert;
        }

        return select(convex, concave, hFace, vFace, surrounded, corner, horiz, vert);
    };
}

function select(convex, concave, hFace, vFace, surrounded, atCorner, atHFace, atVFace) {
    if (atCorner && atHFace && atVFace) {
        return surrounded;
    }
    if (atHFace && atVFace) {
        return concave;
    }
    if (atVFace) {
        return hFace;
    }
    if (atHFace) {
        return vFace;
    }
    return convex;
}
//# sourceMappingURL=computedtilefield.js.map
