'use strict';

var _createClass = require('babel-runtime/helpers/create-class')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _Promise = require('babel-runtime/core-js/promise')['default'];

Object.defineProperty(exports, '__esModule', {
    value: true
});
exports.loadImage = loadImage;
exports.loadIcon = loadIcon;
exports.reloadIcon = reloadIcon;

var _load = require('../load');

var _event = require('../event');

var _is = require('../is');

var Icon = (function () {
    function Icon(image, layout) {
        _classCallCheck(this, Icon);

        this.image = image;
        this.layout = layout;
    }

    _createClass(Icon, [{
        key: 'draw',
        value: function draw(context, x, y, w, h) {
            if (w === 0 || h === 0) {
                return;
            }
            if (!w || !h) {
                return console.warn('Invalid destination size ' + w + ', ' + h);
            }

            var data = this.layout;
            if (!data) {
                return;
            }

            var image = this.image;
            if (!image) {
                return;
            }
            if (!image.width || !image.height) {
                // if missing, Firefox throws and Chrome (sometimes?) has performance issues
                return;
            }

            if (data.solid) {
                context.drawImage(image, data.x, data.y, data.w, data.h, x + data.wFactor * w, y + data.hFactor * h, w * data.sx, h * data.sy);
            } else {
                context.drawImage(image, data.x, data.y, data.w, data.h, x + data.wFactor * w, y + data.hFactor * h, (w + data.insetWRel * w) * data.sx, (h + data.insetHRel * h) * data.sy);
            }
        }

        /*
         * Returns an [x, y, w, h] array of rectangle bounds that specify the area of the canvas that
         * will be taken up when this icon is drawn on a tile of the dimensions given.
         */
    }, {
        key: 'bounds',
        value: function bounds(x, y, w, h) {
            var data = this.layout;
            if (!data) {
                console.warn('Icon has no bounds data because layout not initialized');
                return [x, y, 0, 0];
            } else if (data.solid) {
                return [x + data.wFactor * w, y + data.hFactor * h, w * data.sx, h * data.sy];
            } else {
                return [x + data.wFactor * w, y + data.hFactor * h, (w + data.insetWRel * w) * data.sx, (h + data.insetHRel * h) * data.sy];
            }
        }
    }]);

    return Icon;
})();

exports.Icon = Icon;

function imageSrcPromise(image, src, originalSource) {
    image.src = src;
    return new _Promise(function (resolve, reject) {
        image.onload = function () {
            resolve();
        };
        image.onerror = function (_meaninglessError) {
            // The error parameter doesn't contain details, so just replacing it.
            reject(new Error('Failed to load image from \'' + originalSource + '\''));
        };
    });
}

function dataPromiseToImage(dataPromise, source) {
    var image = new window.Image();

    var promise = dataPromise.then(function (data) {
        return imageSrcPromise(image, data, source);
    })['catch'](function (_error) {
        // HTTP request failed; may be running on localhost, so try loading without HTTP.
        return imageSrcPromise(image, source, source);
    });

    return { image: image, promise: promise };
}

function loadImage(state, source, sizeBytes) {
    var _loadData = (0, _load.loadData)(state, source, sizeBytes, 'image');

    var promise = _loadData.promise;
    var loadingID = _loadData.loadingID;

    var image = undefined;

    var _dataPromiseToImage = dataPromiseToImage(promise, source);

    image = _dataPromiseToImage.image;
    promise = _dataPromiseToImage.promise;

    (0, _load.finalizeLoadingPromise)(state, loadingID, promise);
    return image;
}

function loadIcon(state, source, ax, ay, diameter, sizeBytes) {
    var icon = new Icon();

    if ((0, _is.isString)(source) || source instanceof window.URL) {
        (function () {
            var _loadData2 = (0, _load.loadData)(state, source, sizeBytes, 'image');

            var promise = _loadData2.promise;
            var loadingID = _loadData2.loadingID;

            var image = undefined;

            var _dataPromiseToImage2 = dataPromiseToImage(promise, source);

            image = _dataPromiseToImage2.image;
            promise = _dataPromiseToImage2.promise;

            icon.image = image;

            promise = promise.then(function () {
                icon.layout = computeLayout(ax, ay, diameter, image.width, image.height, 0, 0, image.width, image.height, true);
            });

            (0, _load.finalizeLoadingPromise)(state, loadingID, promise);
        })();
    } else {
        // Assuming source parameter is a drawable object so just use it directly.
        icon.image = source;
        icon.layout = computeLayout(ax, ay, diameter, source.width, source.height, 0, 0, source.width, source.height, true);
    }

    return icon;
}

function reloadIcon(state, icon, source, ax, ay, diameter) {
    var loadId = undefined;
    if (state.skid.load) {
        loadId = (0, _load.startLoading)(state, 0);
    }

    var _dataPromiseToImage3 = dataPromiseToImage((0, _load.reloadData)(state, source, 'image'), source);

    var image = _dataPromiseToImage3.image;
    var promise = _dataPromiseToImage3.promise;

    promise = promise.then(function () {
        // TODO: Putting this line here instead of outside of the promise reduces flicker but also
        // introduces a race condition.
        // TODO: Need a way to mark an image load as canceled.
        icon.image = image;

        icon.layout = computeLayout(ax, ay, diameter, image.width, image.height, 0, 0, image.width, image.height, true);

        if (loadId) {
            (0, _load.doneLoading)(state, loadId);
        }
        (0, _event.handle)(state, 'icon_reloaded', icon);
        (0, _event.handle)(state, 'request_draw');
    });
    (0, _load.finalizeLoadingPromise)(state, undefined, promise);
}

function computeLayout(ax, ay, diameter, untrimmedWidth, untrimmedHeight, sourceX, sourceY, sourceW, sourceH, solid, localX, localY, localW, localH) {
    var layout = {};
    layout.x = sourceX;
    layout.y = sourceY;
    layout.w = sourceW;
    layout.h = sourceH;

    var axRel = undefined;
    if (isNaN(ax)) {
        axRel = 0.5;
    } else {
        axRel = ax / untrimmedWidth;
    }

    var ayRel = undefined;
    if (isNaN(ay)) {
        ayRel = 0.5;
    } else {
        ayRel = ay / untrimmedHeight;
    }

    if (isNaN(diameter)) {
        // fill square tile if no diameter is specified
        if (untrimmedHeight > untrimmedWidth) {
            layout.sx = 1;
            layout.sy = untrimmedHeight / untrimmedWidth;
        } else {
            layout.sy = 1;
            layout.sx = untrimmedWidth / untrimmedHeight;
        }
    } else {
        layout.sx = untrimmedWidth / diameter;
        layout.sy = untrimmedHeight / diameter;
    }

    if (solid) {
        layout.solid = true;
        layout.wFactor = layout.sx * -axRel;
        layout.hFactor = layout.sy * -ayRel;
    } else {
        // Corrects for seams in tile fields caused by precision loss
        // TODO: experiment with splitting numerator from denominator instead
        if (localX % 2 !== 0) {
            localX -= 1;
            localW += 1;
        }
        if (localY % 2 !== 0) {
            localY -= 1;
            localH += 1;
        }

        layout.insetWRel = (localW - untrimmedWidth) / untrimmedWidth;
        layout.insetHRel = (localH - untrimmedHeight) / untrimmedHeight;

        var insetXRel = localX / untrimmedWidth;
        var insetYRel = localY / untrimmedHeight;

        layout.wFactor = layout.sx * (insetXRel - axRel);
        layout.hFactor = layout.sy * (insetYRel - ayRel);
    }

    return layout;
}
//# sourceMappingURL=icon.js.map
