'use strict';

var _createClass = require('babel-runtime/helpers/create-class')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _Object$create = require('babel-runtime/core-js/object/create')['default'];

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

Object.defineProperty(exports, '__esModule', {
    value: true
});

var _cache = require('./cache');

var _group = require('./group');

var TILES_PER_SEGMENT = 6;

var TileField = (function () {
    function TileField(root, tileSize, imageSmoothingEnabled) {
        _classCallCheck(this, TileField);

        this.root = root;
        this._tileSize = tileSize;
        this._rows = _Object$create(null);
        this._segments = _Object$create(null);
        this.layerOffset = 0;

        // In the current imlementation, altering this field after a row has been created will have
        // no effect on that row.
        this.imageSmoothingEnabled = imageSmoothingEnabled;
    }

    _createClass(TileField, [{
        key: '_keyOf',
        value: function _keyOf(x, y) {
            return Math.floor(x / TILES_PER_SEGMENT) + ',' + Math.floor(y);
        }
    }, {
        key: '_ensureRow',
        value: function _ensureRow(y) {
            var result = this._rows[y];
            if (!result) {
                result = new _group.Group(this.root);
                result.layer = y + this.layerOffset;
                this._rows[y] = result;
            }
            return result;
        }
    }, {
        key: 'nodeAt',
        value: function nodeAt(x, y) {
            var key = this._keyOf(x, y);
            var result = this._segments[key];
            if (!result) {
                result = new _cache.Cache(this._ensureRow(y), this._tileSize, this.imageSmoothingEnabled);

                // This will be different depending on what order in which the avatars are added to the
                // segment but the sorting order compared to other segments on the row will be the same
                // regardless because their ranges don't overlap.
                result.layer = x;

                this._segments[key] = result;
            }
            return result;
        }
    }, {
        key: 'clear',
        value: function clear() {
            var _this = this;

            _Object$keys(this._rows).forEach(function (key) {
                return _this._rows[key].remove();
            });
            this._rows = _Object$create(null);
            this._segments = _Object$create(null);
        }
    }]);

    return TileField;
})();

exports.TileField = TileField;
//# sourceMappingURL=tilefield.js.map
