'use strict';

Object.defineProperty(exports, '__esModule', {
    value: true
});
exports.makeViewport = makeViewport;

var _group = require('./group');

var _is = require('../is');

var _event = require('../event');

function makeViewport(state, canvas) {
    var root = new _group.Group();
    state.skid.viewport = { root: root, canvas: canvas };
    return root;
}

(0, _event.addHandler)('request_draw', function (state) {
    if (state.skid.willDraw) {
        return;
    }
    state.skid.willDraw = true;
    if ((0, _is.isNullish)(state.skid.lastFrame)) {
        state.skid.lastFrame = Date.now();
    }

    if ((0, _is.isNullish)(state.skid.drawFunc)) {
        state.skid.drawFunc = function () {
            var currentFrame = Date.now();
            (0, _event.handle)(state, 'before_draw', currentFrame - state.skid.lastFrame);
            var viewport = state.skid.viewport;
            if (viewport && viewport.canvas) {
                var context = viewport.canvas.getContext('2d');
                viewport.root.draw(context);
            }
            state.skid.willDraw = false;
            (0, _event.handle)(state, 'after_draw');
            if (state.skid.willDraw) {
                state.skid.lastFrame = currentFrame;
            } else {
                state.skid.lastFrame = undefined;
            }
        };
    }

    animationFrame(state.skid.drawFunc);
});

(0, _event.addHandler)('load_done', function (state) {
    if (typeof window !== 'undefined') {
        // for unit testing in Node
        window.addEventListener('resize', function () {
            return (0, _event.handle)(state, 'request_draw');
        });
    }
});

var func = undefined;
if (typeof window === 'undefined') {
    // for unit testing in Node
    func = function (callback) {
        return setTimeout(callback, 1000 / 60);
    };
} else {
    func = window.requestAnimationFrame || window.webkitRequestAnimationFrame || window.mozRequestAnimationFrame || window.oRequestAnimationFrame || window.msRequestAnimationFrame;
    if (func) {
        func = func.bind(window);
    } else {
        func = function (callback) {
            return window.setTimeout(callback, 1000 / 60);
        };
    }
}

var animationFrame = func;
//# sourceMappingURL=viewport.js.map
