'use strict';

Object.defineProperty(exports, '__esModule', {
    value: true
});
exports.initPreloader = initPreloader;

var _scenePieavatar = require('../scene/pieavatar');

var _sceneTranslation = require('../scene/translation');

var _sceneTextavatar = require('../scene/textavatar');

var _event = require('../event');

function initPreloader(state, camera) {
    var fillStyle = arguments.length <= 2 || arguments[2] === undefined ? 'black' : arguments[2];

    var meter = new _scenePieavatar.PieAvatar(state, camera);
    meter.layer = 1;
    meter.x.setTo(0.5);
    meter.y.setTo(camera.h.curr / 2);
    meter.w.setTo(0.03);
    meter.h.setTo(0.03);
    meter.fillStyle = fillStyle;
    meter.innerRadiusRel.setTo(0.6);

    var textPosition = new _sceneTranslation.Translation(state, camera);
    textPosition.layer = 1;
    textPosition.x.setTo(0.5);
    textPosition.y.setTo(camera.h.curr / 2 + 0.032);

    var text = new _sceneTextavatar.TextAvatar(state, textPosition, camera);
    text.textAlign = 'center';
    text.textBaseline = 'top';
    text.fillStyle = fillStyle;
    text.font = '18px sans';
    text.text = 'Loading...';

    state.skid.preloader = { meter: meter, textPosition: textPosition, text: text };
}

(0, _event.addHandler)('load_progress', function (state, progress) {
    if (!state.skid.preloader) {
        return;
    }
    state.skid.preloader.meter.breadth.setTo(-progress);
});

(0, _event.addHandler)('load_error', function (state) {
    if (!state.skid.preloader) {
        return;
    }
    state.skid.preloader.meter.fillStyle = '#b00';
    state.skid.preloader.text.fillStyle = '#b00';
    state.skid.preloader.text.text = 'Error';
    (0, _event.handle)(state, 'request_draw');
});

(0, _event.addHandler)('load_done', function (state) {
    if (!state.skid.preloader) {
        return;
    }
    state.skid.preloader.meter.remove();
    state.skid.preloader.textPosition.remove();
    state.skid.preloader.text.remove();
    state.skid.preloader = undefined;
    (0, _event.handle)(state, 'request_draw');
});
//# sourceMappingURL=basicpreloader.js.map
